import click

from leak import main


@click.command()
@click.version_option(
    prog_name="leak",
    package_name="leak",
)
@click.argument(
    "package_name",
    required=True,
)
def cli(package_name):
    """Shows all releases for a package and some info about it.

    PACKAGE_NAME Name of the package to fetch info about.
    """
    main.main(package_name=package_name)


if __name__ == "__main__":
    cli()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['leak']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.4,<9.0.0', 'requests>=2.27.1,<3.0.0', 'rich>=11.2.0,<12.0.0']

entry_points = \
{'console_scripts': ['leak = leak.cli:cli']}

setup_kwargs = {
    'name': 'leak',
    'version': '1.4.0',
    'description': 'Show release information about packages on PyPI',
    'long_description': '## leak\n\n![PyPI](https://img.shields.io/pypi/v/leak?style=flat-square)\n![PyPI - Downloads](https://img.shields.io/pypi/dm/leak?style=flat-square)\n![PyPI - Wheel](https://img.shields.io/pypi/wheel/leak?style=flat-square)\n![PyPI - License](https://img.shields.io/pypi/l/leak?style=flat-square)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/leak?style=flat-square)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg?style=flat-square)](https://github.com/psf/black)\n![Unittests](https://github.com/bmwant/leak/actions/workflows/unittests.yml/badge.svg)\n\nShow info about package releases on PyPI.\n\nIf you need to install specific version of package it is useful to know all available versions to have a choice.\n\nJust run\n\n```bash\n$ leak <package_name>\n# e.g.\n$ leak pyramid\n```\n\nand you will see all releases and some useful statistic about package specified. It will show most recent version, most popular (with highest number of downloads) and some additional information.\n\n### How to install\n\nInstall using pip\n\n```bash\n$ pip install leak\n\n# or to make sure the proper interpreter is used\n$ python -m pip install leak\n```\n\nor upgrade existing version\n\n```bash\n$ pip install --upgrade leak\n\n# or with pip invoked as a module\n$ python -m pip install --upgrade leak\n$ leak --version\n```\n\n### Contribution\n\nSee [DEVELOP.md](./DEVELOP.md) to setup your local development environment and create pull request to this repository once new feature is ready.\n\n### License\n\nDistributed under [MIT License](https://tldrlegal.com/license/mit-license).\n\n### Acknowledgement\n\n🇺🇦 🇺🇦 🇺🇦 We would also thank the Armed Forces of Ukraine for providing security to perform this work. This work has become possible only because of resilience and courage of the Ukrainian Army.\n',
    'author': 'Misha Behersky',
    'author_email': 'bmwant@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/bmwant/leak',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)



import sys
import datetime
import functools
from packaging.version import parse as parse_version

import requests
from rich.console import Console

from leak import config, logger
from leak.version_parser import versions_split


console = Console(highlight=False)
rprint = console.print


def get_latest_time_for_release(release):
    latest_time = config.EPOCH_BEGIN
    date_format = "%Y-%m-%dT%H:%M:%S"
    for release_record in release:
        upload_time = datetime.datetime.strptime(
            release_record["upload_time"], date_format
        )
        if upload_time > latest_time:
            latest_time = upload_time
    return latest_time


def get_max_downloads_for_release(release):
    max_downloads = 0
    for release_record in release:
        if release_record["downloads"] > max_downloads:
            max_downloads = release_record["downloads"]
    return max_downloads


def show_package_info(data):
    def print_row(caption_text, value):
        caption = caption_text.ljust(config.FIRST_COLUMN_LENGTH)
        print("%s %s" % (caption, value))

    info = data["info"]
    name = info["name"]
    summary = info["summary"]
    author = info["author"]
    author_email = info["author_email"]
    url = info["home_page"]
    versions_count = len(data["releases"])

    print("=" * 80)

    rprint(f"[bold bright_white]{name}[/]")
    print(summary)

    print("-" * 80)

    print_row("Author:", author)
    print_row("Author's email:", author_email)
    print_row("Available versions:", versions_count)
    print_row("Home page:", url)

    print("=" * 80)


def get_package_data(package_name):
    url = f"https://pypi.org/pypi/{package_name}/json"
    resp = requests.get(url)
    if resp.status_code != 200:
        raise ValueError("No such package")

    data = resp.json()
    return data


def parse_packages_from_html(html_content):
    return ""


def search_for_package(package_name):
    url = f"https://pypi.python.org/pypi?:action=search&term={package_name}"
    resp = requests.get(url)
    if resp.status_code != 200:
        return []
    return parse_packages_from_html(resp.text)


def main(package_name=""):
    try:
        package_data = get_package_data(package_name)
    except ValueError as e:
        logger.error(e)
        rprint(f"No such package [bold red]{package_name}[/]")
        return sys.exit(1)

    releases = package_data["releases"]
    show_package_info(package_data)
    most_popular_count = 0
    most_popular_release = None
    most_recent_release = None
    most_recent_date = config.EPOCH_BEGIN

    try:
        versions = sorted(releases.keys(), reverse=True, key=parse_version)
    except ValueError as e:
        logger.debug(e)
        logger.debug("Trying to sort versions as strings")
        splitter = functools.partial(versions_split, type_applyer=str)
        versions = sorted(releases.keys(), reverse=True, key=splitter)

    for release_num, release_data in releases.items():
        downloads_count = get_max_downloads_for_release(release_data)
        if downloads_count > most_popular_count:
            most_popular_count = downloads_count
            most_popular_release = release_num

        upload_date = get_latest_time_for_release(release_data)
        if upload_date > most_recent_date:
            most_recent_date = upload_date
            most_recent_release = release_num

    for version in versions:
        if version == most_popular_release == most_recent_release:
            rprint(f"[green]({version}) Most popular and recent. Use this one.[/]")
        elif version == most_popular_release:
            rprint(
                f"[yellow]({version}) Most popular: {most_popular_count} downloads.[/]"
            )
        elif version == most_recent_release:
            release_date = most_recent_date.strftime(config.DATE_FORMAT)
            rprint(f"[cyan]({version}) Most recent: {release_date} release date.[/]")
        else:
            rprint(f"({version})")
